{
  "columns": [
    {
      "aid": "67",
      "attribute": "Name"
    },
    {
      "aid": "67",
      "attribute": "Id"
    }
  ],
  "where": [
    {
      "condition": {
        "aid": "67",
        "attribute": "Project",
        "longlongArray": {
          "values": [
            "3"
          ]
        }
      }
    }
  ],
  "rowLimit": "5"
}